// Copyright (c) 2009-2011, Tor M. Aamodt, Wilson W.L. Fung,
// The University of British Columbia
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimer.
// Redistributions in binary form must reproduce the above copyright notice, this
// list of conditions and the following disclaimer in the documentation and/or
// other materials provided with the distribution.
// Neither the name of The University of British Columbia nor the names of its
// contributors may be used to endorse or promote products derived from this
// software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "option_parser.h"
#include <map>
#include <iostream>

#ifndef ADDRDEC_H
#define ADDRDEC_H

#include "abstract_hardware_model.h"

struct addrdec_t {
   void print( FILE *fp ) const;
    
   unsigned chip;
   unsigned bk;
   unsigned row;
   unsigned col;
   unsigned burst;

   bool operator < (const addrdec_t& temp) const{
	return (chip<temp.chip || bk<temp.bk || row<temp.row || col<temp.col || burst<temp.burst);
	//return(chip<temp.chip);
   };

//   bool operator == (const addrdec_t& temp) const{
//	return (chip==temp.chip && bk==temp.bk && row==temp.row && col==temp.col && burst==temp.burst);
//   };
};


class linear_to_raw_address_translation {
public:
   linear_to_raw_address_translation();
   void addrdec_setoption(option_parser_t opp);
   void init(unsigned int nchips); 

   // accessors
   void addrdec_tlx(new_addr_type addr, addrdec_t *tlx) const; 
   new_addr_type partition_address( new_addr_type addr ) const;

private:
   void addrdec_parseoption(const char *option);

   enum {
      CHIP  = 0,
      BK    = 1,
      ROW   = 2,
      COL   = 3,
      BURST = 4,
      N_ADDRDEC
   };

   const char *addrdec_option;
   int gpgpu_mem_address_mask;

   int ADDR_CHIP_S;
   unsigned char addrdec_mklow[N_ADDRDEC];
   unsigned char addrdec_mkhigh[N_ADDRDEC];
   new_addr_type addrdec_mask[N_ADDRDEC];
   
   unsigned int gap;
   int Nchips;

   mutable std::map<addrdec_t, new_addr_type> addrdec_map;
   
};

#endif
